// Downloaded from https://repo.progsbase.com - Code Developed Using progsbase.

function ZLibCompressNoCompression(data){
  var zlibStruct;

  zlibStruct = {};

  zlibStruct.CMF = 120;
  zlibStruct.FLG = 1;
  zlibStruct.CompressedDataBlocks = DeflateDataNoCompression(data);
  zlibStruct.Adler32CheckValue = ComputeAdler32(data);

  return zlibStruct;
}
function ZLibCompressStaticHuffman(data, level){
  var zlibStruct;

  zlibStruct = {};

  zlibStruct.CMF = 120;
  zlibStruct.FLG = 1;
  zlibStruct.CompressedDataBlocks = DeflateDataStaticHuffman(data, level);
  zlibStruct.Adler32CheckValue = ComputeAdler32(data);

  return zlibStruct;
}
function test(){
  return 0;
}
function ComputeAdler32(data){
  var a, b, m, i;

  a = 1;
  b = 0;
  m = 65521;

  for(i = 0; i < ByteArrayLength(data); i = i + 1){
    a = (a + GetByte(data, i))%m;
    b = (b + a)%m;
  }

  return b*Math.pow(2, 16) + a;
}
function Pack(data, level){
  var output, trimmed;
  var read, i, j, write, start, len, beststart, bestlen, maxforward, size, plain, maxlength;
  var done;

  /* 5-65536 */
  size = level - 1;
  maxlength = 255;

  output = CreateByteArray(Math.ceil(ByteArrayLength(data)*1.1));

  write = 0;
  read = 0;
  plain = 0;
  for(; read < ByteArrayLength(data); ){
    /* Search */
    bestlen = 0;
    beststart = 0;
    /* Lower limit at most 0. */
    start = Math.max(0, read - size);
    for(i = read - 1; i >= start; i = i - 1){
      /* Maximum forward is limited by 1) size, 2) data length or 3) distance to down to search start (i) */
      maxforward = Math.min(maxlength, Math.min(ByteArrayLength(data) - read, read - i));
      len = 0;
      done = false;
      for(j = 0; j < maxforward &&  !done ; j = j + 1){
        if(GetByte(data, i + j) == GetByte(data, read + j)){
          len = len + 1;
        }else{
          done = true;
        }
      }

      if(len > bestlen){
        bestlen = len;
        beststart = read - i;
      }
    }

    /* Write data. */
    if(read + 1 == ByteArrayLength(data)){
      plain = plain + 1;
      read = read + 1;
    }

    if(bestlen >= 4 || plain == 254 || read == ByteArrayLength(data)){
      if(plain > 0){
        SetByte(output, write, plain);
        write = write + 1;
        for(i = 0; i < plain; i = i + 1){
          SetByte(output, write, GetByte(data, read - plain + i));
          write = write + 1;
        }
      }
      plain = 0;
    }

    if(bestlen >= 4){
      if(beststart > 255){
        SetByte(output, write + 0, 0);
        SetByte(output, write + 1, Math.floor(beststart/256));
        SetByte(output, write + 2, beststart%256);
        SetByte(output, write + 3, bestlen);
        write = write + 4;
      }else{
        SetByte(output, write + 0, 255);
        SetByte(output, write + 1, beststart);
        SetByte(output, write + 2, bestlen);
        write = write + 3;
      }

      read = read + bestlen;
    }else{
      plain = plain + 1;
      read = read + 1;
    }
  }

  trimmed = CreateByteArray(write);

  for(i = 0; i < write; i = i + 1){
    SetByte(trimmed, i, GetByte(output, i));
  }

  return trimmed;
}
function Unpack(data){
  var output;
  var read, write, i, dist, lengthx, pos, start, unpackedLength;

  /* Compute length */
  unpackedLength = 0;
  read = 0;
  for(; read < ByteArrayLength(data); ){
    lengthx = GetByte(data, read);

    if(lengthx == 0 || lengthx == 255){
      if(lengthx == 0){
        unpackedLength = unpackedLength + GetByte(data, read + 3);
        read = read + 4;
      }else{
        unpackedLength = unpackedLength + GetByte(data, read + 2);
        read = read + 3;
      }
    }else{
      read = read + 1;
      unpackedLength = unpackedLength + lengthx;
      read = read + lengthx;
    }
  }

  /* Allocate result memory. */
  output = CreateByteArray(unpackedLength);

  /* Unpack */
  write = 0;
  read = 0;
  for(; read < ByteArrayLength(data); ){
    lengthx = GetByte(data, read);

    if(lengthx == 0 || lengthx == 255){
      if(lengthx == 0){
        dist = GetByte(data, read + 1)*256;
        dist = dist + GetByte(data, read + 2);
        lengthx = GetByte(data, read + 3);
        read = read + 4;
      }else{
        dist = GetByte(data, read + 1);
        lengthx = GetByte(data, read + 2);
        read = read + 3;
      }

      start = write - dist;

      for(i = 0; i < lengthx; i = i + 1){
        pos = start + i;
        SetByte(output, write, GetByte(output, pos));
        write = write + 1;
      }
    }else{
      read = read + 1;
      for(i = 0; i < lengthx; i = i + 1){
        SetByte(output, write, GetByte(data, read));
        write = write + 1;
        read = read + 1;
      }
    }
  }

  return output;
}
function DeflateDataStaticHuffman(data, level){
  var bytes;
  var currentBit;
  var i;
  var copy;
  var code, lengthx, compressedCode, lengthAdditionLength, distanceCode;
  var distanceReference, lengthReference, lengthAddition;
  var distanceAdditionReference, distanceAdditionLengthReference;
  var bitReverseLookupTable;
  var match;

  code = CreateNumberReference(0);
  lengthx = CreateNumberReference(0);
  compressedCode = CreateNumberReference(0);
  lengthAdditionLength = CreateNumberReference(0);
  distanceCode = CreateNumberReference(0);
  distanceReference = CreateNumberReference(0);
  lengthReference = CreateNumberReference(0);
  lengthAddition = CreateNumberReference(0);
  distanceAdditionReference = CreateNumberReference(0);
  distanceAdditionLengthReference = CreateNumberReference(0);
  match = {};

  bytes = CreateAndFillByteArray(Math.max(ByteArrayLength(data)*2, 100), 0);
  currentBit = CreateNumberReference(0);

  bitReverseLookupTable = GenerateBitReverseLookupTable(9);

  /* Final block */
  AppendBitsToBytesRight(bytes, currentBit, 1, 1);
  /* Fixed code */
  AppendBitsToBytesRight(bytes, currentBit, 1, 2);

  for(i = 0; i < ByteArrayLength(data); ){
    FindMatch(data, i, distanceReference, lengthReference, match, level);

    if(match.booleanValue){
      GetDeflateLengthCode(lengthReference.numberValue, compressedCode, lengthAddition, lengthAdditionLength);
      GetDeflateDistanceCode(distanceReference.numberValue, distanceCode, distanceAdditionReference, distanceAdditionLengthReference, bitReverseLookupTable);
    }

    if( !match.booleanValue ){
      GetDeflateStaticHuffmanCode(GetByte(data, i), code, lengthx, bitReverseLookupTable);
      AppendBitsToBytesRight(bytes, currentBit, code.numberValue, lengthx.numberValue);
      i = i + 1;
    }else{
      GetDeflateStaticHuffmanCode(compressedCode.numberValue, code, lengthx, bitReverseLookupTable);
      AppendBitsToBytesRight(bytes, currentBit, code.numberValue, lengthx.numberValue);
      AppendBitsToBytesRight(bytes, currentBit, lengthAddition.numberValue, lengthAdditionLength.numberValue);
      AppendBitsToBytesRight(bytes, currentBit, distanceCode.numberValue, 5);
      AppendBitsToBytesRight(bytes, currentBit, distanceAdditionReference.numberValue, distanceAdditionLengthReference.numberValue);
      i = i + lengthReference.numberValue;
    }
  }

  /* Stop symbol */
  GetDeflateStaticHuffmanCode(256, code, lengthx, bitReverseLookupTable);
  AppendBitsToBytesRight(bytes, currentBit, code.numberValue, lengthx.numberValue);

  copy = {};
  CopyByteArrayRange(bytes, 0, Math.ceil(currentBit.numberValue/8), copy);
  delete(bytes);

  return copy;
}
function FindMatch(data, pos, distanceReference, lengthReference, match, level){
  var i, j;
  var deflateMinMength, deflateMaxLength, deflateMaxDistance;
  var longest, maxLength, distanceForMax;
  var startDistance, matchLength;
  var done;

  deflateMinMength = 3;
  deflateMaxLength = 258;

  longest = Math.min(pos - 1, deflateMaxLength);
  longest = Math.min(ByteArrayLength(data) - pos, longest);

  deflateMaxDistance = Math.floor(32768/10*level);

  startDistance = Math.min(pos, deflateMaxDistance);

  if(longest >= deflateMinMength){
    maxLength = 0;
    distanceForMax = 0;

    for(i = pos - 1; i >= pos - startDistance && maxLength != longest; i = i - 1){
      matchLength = 0;
      done = false;
      for(j = 0; j < longest &&  !done ; j = j + 1){
        if(GetByte(data, i + j) == GetByte(data, pos + j)){
          matchLength = matchLength + 1;
        }else{
          done = true;
        }
      }

      if(matchLength >= deflateMinMength && matchLength > maxLength){
        maxLength = matchLength;
        distanceForMax = pos - i;
      }
    }

    if(maxLength >= deflateMinMength){
      match.booleanValue = true;
      lengthReference.numberValue = maxLength;
      distanceReference.numberValue = distanceForMax;
    }else{
      match.booleanValue = false;
    }
  }else{
    match.booleanValue = false;
  }
}
function GenerateBitReverseLookupTable(bits){
  var table;
  var i;

  table = [];
  table.length = Math.pow(2, bits);

  for(i = 0; i < table.length; i = i + 1){
    table[i] = ReverseBits(i, 32);
  }

  return table;
}
function ReverseBits(x, bits){
  var b, bit, i;

  b = 0;

  for(i = 0; i < bits; i = i + 1){
    b = ShiftLeft4Byte(b, 1);
    bit = And4Byte(x, 1);
    b = Or4Byte(b, bit);
    x = ShiftRight4Byte(x, 1);
  }

  return b;
}
function DeflateDataNoCompression(data){
  var deflated;
  var position;
  var block, i, blocks, blocklength, maxblocksize;

  maxblocksize = Math.pow(2, 16) - 1;
  blocks = Math.ceil(ByteArrayLength(data)/maxblocksize);

  position = CreateNumberReference(0);

  deflated = CreateByteArray((1 + 4)*blocks + ByteArrayLength(data));

  for(block = 0; block < blocks; block = block + 1){
    if(block + 1 == blocks){
      WriteByte(deflated, 1, position);
    }else{
      WriteByte(deflated, 0, position);
    }
    blocklength = Math.min(ByteArrayLength(data) - block*maxblocksize, maxblocksize);
    Write2BytesLE(deflated, blocklength, position);
    Write2BytesLE(deflated, Not2Byte(blocklength), position);

    for(i = 0; i < blocklength; i = i + 1){
      WriteByte(deflated, GetByte(data, block*maxblocksize + i), position);
    }
  }

  return deflated;
}
function GetDeflateStaticHuffmanCode(b, code, lengthx, bitReverseLookupTable){
  var reversed;

  if(b >= 0 && b <= 143){
    code.numberValue = 48 + b;
    lengthx.numberValue = 8;
  }else if(b >= 144 && b <= 255){
    code.numberValue = b - 144 + 400;
    lengthx.numberValue = 9;
  }else if(b >= 256 && b <= 279){
    code.numberValue = b - 256 + 0;
    lengthx.numberValue = 7;
  }else if(b >= 280 && b <= 287){
    code.numberValue = b - 280 + 192;
    lengthx.numberValue = 8;
  }

  reversed = bitReverseLookupTable[code.numberValue];
  code.numberValue = ShiftRight4Byte(reversed, 32 - lengthx.numberValue);
}
function GetDeflateLengthCode(lengthx, code, lengthAddition, lengthAdditionLength){
  if(lengthx >= 3 && lengthx <= 10){
    code.numberValue = 257 + lengthx - 3;
    lengthAdditionLength.numberValue = 0;
  }else if(lengthx >= 11 && lengthx <= 18){
    code.numberValue = 265 + Math.floor((lengthx - 11)/2);
    lengthAddition.numberValue = Math.floor((lengthx - 11)%2);
    lengthAdditionLength.numberValue = 1;
  }else if(lengthx >= 19 && lengthx <= 34){
    code.numberValue = 269 + Math.floor((lengthx - 19)/4);
    lengthAddition.numberValue = Math.floor((lengthx - 19)%4);
    lengthAdditionLength.numberValue = 2;
  }else if(lengthx >= 35 && lengthx <= 66){
    code.numberValue = 273 + Math.floor((lengthx - 35)/8);
    lengthAddition.numberValue = Math.floor((lengthx - 35)%8);
    lengthAdditionLength.numberValue = 3;
  }else if(lengthx >= 67 && lengthx <= 130){
    code.numberValue = 277 + Math.floor((lengthx - 67)/16);
    lengthAddition.numberValue = Math.floor((lengthx - 67)%16);
    lengthAdditionLength.numberValue = 4;
  }else if(lengthx >= 131 && lengthx <= 257){
    code.numberValue = 281 + Math.floor((lengthx - 131)/32);
    lengthAddition.numberValue = Math.floor((lengthx - 131)%32);
    lengthAdditionLength.numberValue = 5;
  }else if(lengthx == 258){
    code.numberValue = 285;
    lengthAdditionLength.numberValue = 0;
  }
}
function GetDeflateDistanceCode(distance, code, distanceAdditionReference, distanceAdditionLengthReference, bitReverseLookupTable){
  var reversed;

  if(distance >= 1 && distance <= 4){
    code.numberValue = distance - 1;
    distanceAdditionLengthReference.numberValue = 0;
  }else if(distance >= 5 && distance <= 8){
    code.numberValue = 4 + Math.floor((distance - 5)/2);
    distanceAdditionReference.numberValue = Math.floor((distance - 5)%2);
    distanceAdditionLengthReference.numberValue = 1;
  }else if(distance >= 9 && distance <= 16){
    code.numberValue = 6 + Math.floor((distance - 9)/4);
    distanceAdditionReference.numberValue = Math.floor((distance - 9)%4);
    distanceAdditionLengthReference.numberValue = 2;
  }else if(distance >= 17 && distance <= 32){
    code.numberValue = 8 + Math.floor((distance - 17)/8);
    distanceAdditionReference.numberValue = Math.floor((distance - 17)%8);
    distanceAdditionLengthReference.numberValue = 3;
  }else if(distance >= 33 && distance <= 64){
    code.numberValue = 10 + Math.floor((distance - 33)/16);
    distanceAdditionReference.numberValue = Math.floor((distance - 33)%16);
    distanceAdditionLengthReference.numberValue = 4;
  }else if(distance >= 65 && distance <= 128){
    code.numberValue = 12 + Math.floor((distance - 65)/32);
    distanceAdditionReference.numberValue = Math.floor((distance - 65)%32);
    distanceAdditionLengthReference.numberValue = 5;
  }else if(distance >= 129 && distance <= 256){
    code.numberValue = 14 + Math.floor((distance - 129)/64);
    distanceAdditionReference.numberValue = Math.floor((distance - 129)%64);
    distanceAdditionLengthReference.numberValue = 6;
  }else if(distance >= 257 && distance <= 512){
    code.numberValue = 16 + Math.floor((distance - 257)/128);
    distanceAdditionReference.numberValue = Math.floor((distance - 257)%128);
    distanceAdditionLengthReference.numberValue = 7;
  }else if(distance >= 513 && distance <= 1024){
    code.numberValue = 18 + Math.floor((distance - 513)/256);
    distanceAdditionReference.numberValue = Math.floor((distance - 513)%256);
    distanceAdditionLengthReference.numberValue = 8;
  }else if(distance >= 1025 && distance <= 2048){
    code.numberValue = 20 + Math.floor((distance - 1025)/Math.pow(2, 9));
    distanceAdditionReference.numberValue = Math.floor((distance - 1025)%Math.pow(2, 9));
    distanceAdditionLengthReference.numberValue = 9;
  }else if(distance >= 2049 && distance <= 4096){
    code.numberValue = 22 + Math.floor((distance - 2049)/Math.pow(2, 10));
    distanceAdditionReference.numberValue = Math.floor((distance - 2049)%Math.pow(2, 10));
    distanceAdditionLengthReference.numberValue = 10;
  }else if(distance >= 4097 && distance <= 8192){
    code.numberValue = 24 + Math.floor((distance - 4097)/Math.pow(2, 11));
    distanceAdditionReference.numberValue = Math.floor((distance - 4097)%Math.pow(2, 11));
    distanceAdditionLengthReference.numberValue = 11;
  }else if(distance >= 8193 && distance <= 16384){
    code.numberValue = 26 + Math.floor((distance - 8193)/Math.pow(2, 12));
    distanceAdditionReference.numberValue = Math.floor((distance - 8193)%Math.pow(2, 12));
    distanceAdditionLengthReference.numberValue = 12;
  }else if(distance >= 16385 && distance <= 32768){
    code.numberValue = 28 + Math.floor((distance - 16385)/Math.pow(2, 13));
    distanceAdditionReference.numberValue = Math.floor((distance - 16385)%Math.pow(2, 13));
    distanceAdditionLengthReference.numberValue = 13;
  }

  reversed = bitReverseLookupTable[code.numberValue];
  code.numberValue = ShiftRight4Byte(reversed, 32 - 5);
}
function AppendBitsToBytesLeft(bytes, nextbit, data, lengthx){
  var bytePos, bitPos, segment, part, remove;

  for(; lengthx > 0; ){
    bytePos = Truncate(nextbit.numberValue/8);
    bitPos = nextbit.numberValue%8;

    if(lengthx < 8 - bitPos){
      part = ShiftLeft4Byte(data, 8 - bitPos - lengthx);

      SetByte(bytes, bytePos, Or4Byte(GetByte(bytes, bytePos), part));

      nextbit.numberValue = nextbit.numberValue + lengthx;

      lengthx = 0;
    }else{
      segment = 8 - bitPos;

      part = ShiftRight4Byte(data, lengthx - segment);
      SetByte(bytes, bytePos, Or4Byte(GetByte(bytes, bytePos), part));
      nextbit.numberValue = nextbit.numberValue + segment;

      remove = ShiftLeft4Byte(part, lengthx - segment);
      data = Xor4Byte(data, remove);

      lengthx = lengthx - segment;
    }
  }
}
function AppendBitsToBytesRight(bytes, nextbit, data, lengthx){
  var bytePos, bitPos, segment, part;
  var mask;

  for(; lengthx > 0; ){
    bytePos = Truncate(nextbit.numberValue/8);
    bitPos = nextbit.numberValue%8;

    if(lengthx < 8 - bitPos){
      part = ShiftLeft4Byte(data, bitPos);

      SetByte(bytes, bytePos, Or4Byte(GetByte(bytes, bytePos), part));

      nextbit.numberValue = nextbit.numberValue + lengthx;

      lengthx = 0;
    }else{
      segment = 8 - bitPos;

      mask = 1;
      mask = ShiftLeft4Byte(mask, segment);
      mask = mask - 1;

      part = And4Byte(mask, data);
      part = ShiftLeft4Byte(part, bitPos);
      SetByte(bytes, bytePos, Or4Byte(GetByte(bytes, bytePos), part));
      nextbit.numberValue = nextbit.numberValue + segment;

      data = ShiftRight4Byte(data, segment);

      lengthx = lengthx - segment;
    }
  }
}
function ReadXbytes(data, position, lengthx){
  var r;
  var i;

  r = CreateByteArray(lengthx);

  for(i = 0; i < lengthx; i = i + 1){
    r.bytes[i] = ReadByte(data, position);
  }

  return r;
}
function Read4bytesBE(data, position){
  var r;

  r = 0;
  r = r + Math.pow(2, 24)*ReadByte(data, position);
  r = r + Math.pow(2, 16)*ReadByte(data, position);
  r = r + Math.pow(2, 8)*ReadByte(data, position);
  r = r + ReadByte(data, position);

  return r;
}
function Read2bytesBE(data, position){
  var r;

  r = 0;
  r = r + Math.pow(2, 8)*ReadByte(data, position);
  r = r + ReadByte(data, position);

  return r;
}
function ReadByte(data, position){
  var next;

  next = data.bytes[position.numberValue];
  position.numberValue = position.numberValue + 1;

  return next;
}
function Read4bytesLE(data, position){
  var r;

  r = 0;
  r = r + ReadByte(data, position);
  r = r + Math.pow(2, 8)*ReadByte(data, position);
  r = r + Math.pow(2, 16)*ReadByte(data, position);
  r = r + Math.pow(2, 24)*ReadByte(data, position);

  return r;
}
function WriteByte(data, b, position){
  data.bytes[position.numberValue] = b;
  position.numberValue = position.numberValue + 1;
}
function Write2BytesLE(data, b, position){
  data.bytes[position.numberValue] = BytesRound(b%Math.pow(2, 8));
  position.numberValue = position.numberValue + 1;
  data.bytes[position.numberValue] = Math.floor(b/Math.pow(2, 8))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
}
function Write4BytesLE(data, b, position){
  data.bytes[position.numberValue] = BytesRound(b%Math.pow(2, 8));
  position.numberValue = position.numberValue + 1;
  data.bytes[position.numberValue] = Math.floor(b/Math.pow(2, 8))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
  data.bytes[position.numberValue] = Math.floor(b/Math.pow(2, 16))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
  data.bytes[position.numberValue] = Math.floor(b/Math.pow(2, 24))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
}
function Write2BytesBE(data, b, position){
  data.bytes[position.numberValue] = Math.floor(b/Math.pow(2, 8))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
  data.bytes[position.numberValue] = BytesRound(b%Math.pow(2, 8));
  position.numberValue = position.numberValue + 1;
}
function Write4BytesBE(data, b, position){
  data.bytes[position.numberValue] = Math.floor(b/Math.pow(2, 24))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
  data.bytes[position.numberValue] = Math.floor(b/Math.pow(2, 16))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
  data.bytes[position.numberValue] = Math.floor(b/Math.pow(2, 8))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
  data.bytes[position.numberValue] = BytesRound(b%Math.pow(2, 8));
  position.numberValue = position.numberValue + 1;
}
function WriteStringBytes(data, cs, position){
  var i, v;

  for(i = 0; i < cs.length; i = i + 1){
    v = cs[i].charCodeAt(0);
    WriteByte(data, v, position);
  }
}
function BytesRound(x){
  return Math.floor(x + 0.5);
}
function ByteArrayToNumberArray(src){
  var dst;
  var i, len;

  len = src.bytes.length;
  dst = [];
  dst.length = len;
  for(i = 0; i < len; i = i + 1){
    dst[i] = src.bytes[i];
  }

  return dst;
}
function NumberArrayToByteArray(src){
  var dst;
  var i, len;

  len = src.length;
  dst = {};
  dst.bytes = [];
  dst.bytes.length = len;
  for(i = 0; i < len; i = i + 1){
    dst.bytes[i] = src[i];
  }

  return dst;
}
function ByteArraysEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.bytes.length == b.bytes.length){
    for(i = 0; i < a.bytes.length && equal; i = i + 1){
      if(a.bytes[i] != b.bytes[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function CopyByteArray(a){
  var i;
  var n;

  n = {};
  n.bytes = [];
  n.bytes.length = a.bytes.length;

  for(i = 0; i < a.bytes.length; i = i + 1){
    n.bytes[i] = a.bytes[i];
  }

  return n;
}
function ByteArrayLength(response){
  return response.bytes.length;
}
function CreateAndFillByteArray(lengthx, value){
  var bytes;
  var i;

  bytes = {};
  bytes.bytes = [];
  bytes.bytes.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    bytes.bytes[i] = value;
  }

  return bytes;
}
function CreateByteArray(lengthx){
  var bytes;

  bytes = {};
  bytes.bytes = [];
  bytes.bytes.length = lengthx;

  return bytes;
}
function SetByte(array, index, value){
  array.bytes[index] = value;
}
function GetByte(array, index){
  return array.bytes[index];
}
function AssertByteArraysEqual(a, b, failures){
  var i;

  if(a.bytes.length == b.bytes.length){
    for(i = 0; i < a.bytes.length; i = i + 1){
      if(GetByte(a, i) != GetByte(b, i)){
        failures.numberValue = failures.numberValue + 1;
      }
    }
  }else{
    failures.numberValue = failures.numberValue + 1;
  }
}
function FreeByteArray(byteArray){
  delete(byteArray.bytes);
  delete(byteArray);
}
function CopyByteArrayRange(a, from, to, b){
  var i, lengthx;
  var n;
  var success;

  if(from >= 0 && from <= a.bytes.length && to >= 0 && to <= a.bytes.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;

    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a.bytes[from + i];
    }

    b.bytes = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function BytesToTextBase16(bytes){
  var comb;
  var i, v;
  var r;

  r = [];
  r.length = bytes.length*2;
  comb = GenerateBase16ByteCombinations();

  for(i = 0; i < bytes.length; i = i + 1){
    v = bytes[i];
    r[2*i + 0] = comb[v].string[0];
    r[2*i + 1] = comb[v].string[1];
  }

  FreeBase64ByteCombinations(comb);

  return r;
}
function TextToBytesBase16(string){
  var comb;
  var i, n, c, d;
  var r;
  var reverseBase16;
  var codes;
  var ch;

  codes = "0123456789ABCDEF".split('');

  reverseBase16 = [];
  reverseBase16.length = 128;
  for(i = 0; i < codes.length; i = i + 1){
    n = codes[i].charCodeAt(0);
    reverseBase16[n] = i;
  }

  if(string.length%2 == 0){
    r = [];
    r.length = string.length/2;
    comb = GenerateBase16ByteCombinations();

    for(i = 0; i < string.length/2; i = i + 1){
      c = string[2*i + 0].charCodeAt(0);
      d = string[2*i + 1].charCodeAt(0);

      n = reverseBase16[c]*16 + reverseBase16[d];

      r[i] = n;
    }

    FreeBase64ByteCombinations(comb);
  }else{
    r = [];
    r.length = 0;
  }

  delete(reverseBase16);

  return r;
}
function FreeBase64ByteCombinations(comb){
  var i;
  for(i = 0; i < 256; i = i + 1){
    FreeStringReference(comb[i]);
  }
  delete(comb);
}
function GenerateBase16ByteCombinations(){
  var comb;
  var codes;
  var i, j;
  comb = [];
  comb.length = 256;

  codes = "0123456789ABCDEF".split('');

  for(i = 0; i < 256; i = i + 1){
    comb[i] = {};
    comb[i].string = [];
    comb[i].string.length = 2;
  }

  for(j = 0; j < 16; j = j + 1){
    for(i = 0; i < 16; i = i + 1){
      comb[j*16 + i].string[1] = codes[i];
      comb[j*16 + i].string[0] = codes[j];
    }
  }

  return comb;
}
function Negate(x){
  return  -x;
}
function Positive(x){
  return  +x;
}
function Factorial(x){
  var i, f;

  f = 1;

  for(i = 2; i <= x; i = i + 1){
    f = f*i;
  }

  return f;
}
function Round(x){
  return Math.floor(x + 0.5);
}
function BankersRound(x){
  var r;

  if(Absolute(x - Truncate(x)) == 0.5){
    if( !DivisibleBy(Round(x), 2) ){
      r = Round(x) - 1;
    }else{
      r = Round(x);
    }
  }else{
    r = Round(x);
  }

  return r;
}
function Ceil(x){
  return Math.ceil(x);
}
function Floor(x){
  return Math.floor(x);
}
function Truncate(x){
  var t;

  if(x >= 0){
    t = Math.floor(x);
  }else{
    t = Math.ceil(x);
  }

  return t;
}
function Absolute(x){
  return Math.abs(x);
}
function Logarithm(x){
  return Math.log10(x);
}
function NaturalLogarithm(x){
  return Math.log(x);
}
function Sin(x){
  return Math.sin(x);
}
function Cos(x){
  return Math.cos(x);
}
function Tan(x){
  return Math.tan(x);
}
function Asin(x){
  return Math.asin(x);
}
function Acos(x){
  return Math.acos(x);
}
function Atan(x){
  return Math.atan(x);
}
function Atan2(y, x){
  var a;

  /* Atan2 is an invalid operation when x = 0 and y = 0, but this method does not return errors. */
  a = 0;

  if(x > 0){
    a = Atan(y/x);
  }else if(x < 0 && y >= 0){
    a = Atan(y/x) + Math.PI;
  }else if(x < 0 && y < 0){
    a = Atan(y/x) - Math.PI;
  }else if(x == 0 && y > 0){
    a = Math.PI/2;
  }else if(x == 0 && y < 0){
    a =  -Math.PI/2;
  }

  return a;
}
function Squareroot(x){
  return Math.sqrt(x);
}
function Exp(x){
  return Math.exp(x);
}
function DivisibleBy(a, b){
  return ((a%b) == 0);
}
function Combinations(n, k){
  var i, j, c;

  c = 1;
  j = 1;
  i = n - k + 1;

  for(; i <= n; ){
    c = c*i;
    c = c/j;

    i = i + 1;
    j = j + 1;
  }

  return c;
}
function Permutations(n, k){
  var i, c;

  c = 1;

  for(i = n - k + 1; i <= n; i = i + 1){
    c = c*i;
  }

  return c;
}
function EpsilonCompare(a, b, epsilon){
  return Math.abs(a - b) < epsilon;
}
function GreatestCommonDivisor(a, b){
  var t;

  for(; b != 0; ){
    t = b;
    b = a%b;
    a = t;
  }

  return a;
}
function GCDWithSubtraction(a, b){
  var g;

  if(a == 0){
    g = b;
  }else{
    for(; b != 0; ){
      if(a > b){
        a = a - b;
      }else{
        b = b - a;
      }
    }

    g = a;
  }

  return g;
}
function IsInteger(a){
  return (a - Math.floor(a)) == 0;
}
function GreatestCommonDivisorWithCheck(a, b, gcdReference){
  var success;
  var gcd;

  if(IsInteger(a) && IsInteger(b)){
    gcd = GreatestCommonDivisor(a, b);
    gcdReference.numberValue = gcd;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function LeastCommonMultiple(a, b){
  var lcm;

  if(a > 0 && b > 0){
    lcm = Math.abs(a*b)/GreatestCommonDivisor(a, b);
  }else{
    lcm = 0;
  }

  return lcm;
}
function Sign(a){
  var s;

  if(a > 0){
    s = 1;
  }else if(a < 0){
    s =  -1;
  }else{
    s = 0;
  }

  return s;
}
function Max(a, b){
  return Math.max(a, b);
}
function Min(a, b){
  return Math.min(a, b);
}
function Power(a, b){
  return Math.pow(a, b);
}
function Gamma(x){
  return LanczosApproximation(x);
}
function LogGamma(x){
  return Math.log(Gamma(x));
}
function LanczosApproximation(z){
  var p;
  var i, y, t, x;

  p = [];
  p.length = 8;
  p[0] = 676.5203681218851;
  p[1] =  -1259.1392167224028;
  p[2] = 771.32342877765313;
  p[3] =  -176.61502916214059;
  p[4] = 12.507343278686905;
  p[5] =  -0.13857109526572012;
  p[6] = 9.9843695780195716e-6;
  p[7] = 1.5056327351493116e-7;

  if(z < 0.5){
    y = Math.PI/(Math.sin(Math.PI*z)*LanczosApproximation(1 - z));
  }else{
    z = z - 1;
    x = 0.99999999999980993;
    for(i = 0; i < p.length; i = i + 1){
      x = x + p[i]/(z + i + 1);
    }
    t = z + p.length - 0.5;
    y = Math.sqrt(2*Math.PI)*Math.pow(t, z + 0.5)*Math.exp( -t)*x;
  }

  return y;
}
function Beta(x, y){
  return Gamma(x)*Gamma(y)/Gamma(x + y);
}
function Sinh(x){
  return (Math.exp(x) - Math.exp( -x))/2;
}
function Cosh(x){
  return (Math.exp(x) + Math.exp( -x))/2;
}
function Tanh(x){
  return Sinh(x)/Cosh(x);
}
function Cot(x){
  return 1/Math.tan(x);
}
function Sec(x){
  return 1/Math.cos(x);
}
function Csc(x){
  return 1/Math.sin(x);
}
function Coth(x){
  return Cosh(x)/Sinh(x);
}
function Sech(x){
  return 1/Cosh(x);
}
function Csch(x){
  return 1/Sinh(x);
}
function Error(x){
  var y, t, tau, c1, c2, c3, c4, c5, c6, c7, c8, c9, c10;

  if(x == 0){
    y = 0;
  }else if(x < 0){
    y =  -Error( -x);
  }else{
    c1 =  -1.26551223;
    c2 =  +1.00002368;
    c3 =  +0.37409196;
    c4 =  +0.09678418;
    c5 =  -0.18628806;
    c6 =  +0.27886807;
    c7 =  -1.13520398;
    c8 =  +1.48851587;
    c9 =  -0.82215223;
    c10 =  +0.17087277;

    t = 1/(1 + 0.5*Math.abs(x));

    tau = t*Math.exp( -(Math.pow(x, 2)) + c1 + t*(c2 + t*(c3 + t*(c4 + t*(c5 + t*(c6 + t*(c7 + t*(c8 + t*(c9 + t*c10)))))))));

    y = 1 - tau;
  }

  return y;
}
function ErrorInverse(x){
  var y, a, t;

  a = (8*(Math.PI - 3))/(3*Math.PI*(4 - Math.PI));

  t = 2/(Math.PI*a) + Math.log(1 - Math.pow(x, 2))/2;
  y = Sign(x)*Math.sqrt(Math.sqrt(Math.pow(t, 2) - Math.log(1 - Math.pow(x, 2))/a) - t);

  return y;
}
function FallingFactorial(x, n){
  var k, y;

  y = 1;

  for(k = 0; k <= n - 1; k = k + 1){
    y = y*(x - k);
  }

  return y;
}
function RisingFactorial(x, n){
  var k, y;

  y = 1;

  for(k = 0; k <= n - 1; k = k + 1){
    y = y*(x + k);
  }

  return y;
}
function Hypergeometric(a, b, c, z, maxIterations, precision){
  var y;

  if(Math.abs(z) >= 0.5){
    y = Math.pow(1 - z,  -a)*HypergeometricDirect(a, c - b, c, z/(z - 1), maxIterations, precision);
  }else{
    y = HypergeometricDirect(a, b, c, z, maxIterations, precision);
  }

  return y;
}
function HypergeometricDirect(a, b, c, z, maxIterations, precision){
  var y, yp, n;
  var done;

  y = 0;
  done = false;

  for(n = 0; n < maxIterations &&  !done ; n = n + 1){
    yp = RisingFactorial(a, n)*RisingFactorial(b, n)/RisingFactorial(c, n)*Math.pow(z, n)/Factorial(n);
    if(Math.abs(yp) < precision){
      done = true;
    }
    y = y + yp;
  }

  return y;
}
function BernouilliNumber(n){
  return AkiyamaTanigawaAlgorithm(n);
}
function AkiyamaTanigawaAlgorithm(n){
  var m, j, B;
  var A;

  A = [];
  A.length = n + 1;

  for(m = 0; m <= n; m = m + 1){
    A[m] = 1/(m + 1);
    for(j = m; j >= 1; j = j - 1){
      A[j - 1] = j*(A[j - 1] - A[j]);
    }
  }

  B = A[0];

  delete(A);

  return B;
}
function CreateBooleanReference(value){
  var ref;

  ref = {};
  ref.booleanValue = value;

  return ref;
}
function CreateBooleanArrayReference(value){
  var ref;

  ref = {};
  ref.booleanArray = value;

  return ref;
}
function CreateBooleanArrayReferenceLengthValue(lengthx, value){
  var ref;
  var i;

  ref = {};
  ref.booleanArray = [];
  ref.booleanArray.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.booleanArray[i] = value;
  }

  return ref;
}
function FreeBooleanArrayReference(booleanArrayReference){
  delete(booleanArrayReference.booleanArray);
  delete(booleanArrayReference);
}
function CreateCharacterReference(value){
  var ref;

  ref = {};
  ref.characterValue = value;

  return ref;
}
function CreateNumberReference(value){
  var ref;

  ref = {};
  ref.numberValue = value;

  return ref;
}
function CreateNumberArrayReference(value){
  var ref;

  ref = {};
  ref.numberArray = value;

  return ref;
}
function CreateNumberArrayReferenceLengthValue(lengthx, value){
  var ref;
  var i;

  ref = {};
  ref.numberArray = [];
  ref.numberArray.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.numberArray[i] = value;
  }

  return ref;
}
function FreeNumberArrayReference(numberArrayReference){
  delete(numberArrayReference.numberArray);
  delete(numberArrayReference);
}
function CreateStringReference(value){
  var ref;

  ref = {};
  ref.string = value;

  return ref;
}
function CreateStringReferenceLengthValue(lengthx, value){
  var ref;
  var i;

  ref = {};
  ref.string = [];
  ref.string.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.string[i] = value;
  }

  return ref;
}
function FreeStringReference(stringReference){
  delete(stringReference.string);
  delete(stringReference);
}
function CreateStringArrayReference(strings){
  var ref;

  ref = {};
  ref.stringArray = strings;

  return ref;
}
function CreateStringArrayReferenceLengthValue(lengthx, value){
  var ref;
  var i;

  ref = {};
  ref.stringArray = [];
  ref.stringArray.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.stringArray[i] = CreateStringReference(value);
  }

  return ref;
}
function FreeStringArrayReference(stringArrayReference){
  var i;

  for(i = 0; i < stringArrayReference.stringArray.length; i = i + 1){
    delete(stringArrayReference.stringArray[i]);
  }
  delete(stringArrayReference.stringArray);
  delete(stringArrayReference);
}
function And4Byte(n1, n2){
  return AndBytes(n1, n2, 4);
}
function And2Byte(n1, n2){
  return AndBytes(n1, n2, 2);
}
function AndByte(n1, n2){
  return AndBytes(n1, n2, 1);
}
function AndBytes(n1, n2, bytes){
  var byteVal, result, i;

  byteVal = 1;
  result = 0;

  if(n1 >= 0 && n1 < Math.pow(2, bytes*8) && n2 >= 0 && n2 < Math.pow(2, bytes*8)){
    n1 = Truncate(n1);
    n2 = Truncate(n2);
    bytes = Truncate(bytes);

    for(i = 0; i < bytes*8; i = i + 1){
      if(n1%2 == 1 && n2%2 == 1){
        result = result + byteVal;
      }
      n1 = Math.floor(n1/2);
      n2 = Math.floor(n2/2);
      byteVal = byteVal*2;
    }
  }

  return result;
}
function Or4Byte(n1, n2){
  return OrBytes(n1, n2, 4);
}
function Or2Byte(n1, n2){
  return OrBytes(n1, n2, 2);
}
function OrByte(n1, n2){
  return OrBytes(n1, n2, 1);
}
function OrBytes(n1, n2, bytes){
  var byteVal, result, i;

  byteVal = 1;
  result = 0;

  if(n1 >= 0 && n1 < Math.pow(2, bytes*8) && n2 >= 0 && n2 < Math.pow(2, bytes*8)){
    n1 = Truncate(n1);
    n2 = Truncate(n2);
    bytes = Truncate(bytes);

    for(i = 0; i < bytes*8; i = i + 1){
      if(n1%2 == 1 || n2%2 == 1){
        result = result + byteVal;
      }
      n1 = Math.floor(n1/2);
      n2 = Math.floor(n2/2);
      byteVal = byteVal*2;
    }
  }

  return result;
}
function Xor4Byte(n1, n2){
  return XorBytes(n1, n2, 4);
}
function Xor2Byte(n1, n2){
  return XorBytes(n1, n2, 2);
}
function XorByte(n1, n2){
  return XorBytes(n1, n2, 1);
}
function XorBytes(n1, n2, bytes){
  var byteVal, result, i;

  byteVal = 1;
  result = 0;

  if(n1 >= 0 && n1 < Math.pow(2, bytes*8) && n2 >= 0 && n2 < Math.pow(2, bytes*8)){
    n1 = Truncate(n1);
    n2 = Truncate(n2);
    bytes = Truncate(bytes);

    for(i = 0; i < bytes*8; i = i + 1){
      if(n1%2 != n2%2){
        result = result + byteVal;
      }
      n1 = Math.floor(n1/2);
      n2 = Math.floor(n2/2);
      byteVal = byteVal*2;
    }
  }

  return result;
}
function Not4Byte(b){
  return NotBytes(b, 4);
}
function Not2Byte(b){
  return NotBytes(b, 2);
}
function NotByte(b){
  return NotBytes(b, 1);
}
function NotBytes(b, lengthx){
  var result;

  result = 0;

  if(b >= 0 && b < Math.pow(2, lengthx*8)){
    b = Truncate(b);
    lengthx = Truncate(lengthx);

    result = Math.pow(2, lengthx*8) - b - 1;
  }

  return result;
}
function ShiftLeft4Byte(b, amount){
  return ShiftLeftBytes(b, amount, 4);
}
function ShiftLeft2Byte(b, amount){
  return ShiftLeftBytes(b, amount, 2);
}
function ShiftLeftByte(b, amount){
  return ShiftLeftBytes(b, amount, 1);
}
function ShiftLeftBytes(b, amount, lengthx){
  var result;

  result = 0;

  if(b >= 0 && b < Math.pow(2, lengthx*8) && amount >= 0 && amount <= lengthx*8){
    b = Truncate(b);
    amount = Truncate(amount);

    result = b*Math.pow(2, amount);
  }

  return result;
}
function ShiftRight4Byte(b, amount){
  return ShiftRightBytes(b, amount, 4);
}
function ShiftRight2Byte(b, amount){
  return ShiftRightBytes(b, amount, 2);
}
function ShiftRightByte(b, amount){
  return ShiftRightBytes(b, amount, 1);
}
function ShiftRightBytes(b, amount, lengthx){
  var result;

  result = 0;

  if(b >= 0 && b < Math.pow(2, lengthx*8) && amount >= 0 && amount <= lengthx*8){
    b = Truncate(b);
    amount = Truncate(amount);

    result = Truncate(b/Math.pow(2, amount));
  }

  return result;
}
function RotateLeft4Byte(w, n){
  var a;
  /*return (w << n) | (w >> (32 - n)); */
  /* Mask the upper bits first, then rotate. */
  a = And4Byte(w, Not4Byte(ShiftLeft4Byte(1, n) - 1));
  return Or4Byte(ShiftLeft4Byte(a, n), ShiftRight4Byte(w, (32 - n)));
}
function RotateRight4Bytes(w, n){
  var a;
  /* return (w >> d) | (w << (32 - n)); */
  /* Mask away the upper bits first, then perform the shift. */
  a = And4Byte(w, ShiftLeft4Byte(1, n) - 1);
  return Or4Byte(ShiftRight4Byte(w, n), ShiftLeft4Byte(a, 32 - n));
}
function CreateBooleanArrayFromNumber(w, size){
  var out;
  var p, j;

  out = arraysCreateBooleanArray(size, false);

  j = 0;
  p = 1;
  for(; p < w; ){
    p = p*2;
    j = j + 1;
  }

  for(; j >= 0; j = j - 1){
    if(w >= p){
      w = w - p;
      if(j < size){
        out[size - 1 - j] = true;
      }
    }
    p = p/2;
  }

  return out;
}
function BooleanArrayToNumber(bits){
  var w, i, p;

  w = 0;
  p = 1;
  for(i = 31; i >= 0; i = i - 1){
    if(bits[i]){
      w = w + p;
    }
    p = p*2;
  }

  return w;
}
function BooleanAnd(a, b){
  var out;
  var i, lengthx;

  lengthx = a.length;

  out = [];
  out.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    out[i] = a[i] && b[i];
  }
  return out;
}
function BooleanXor(a, b){
  var out;
  var i, lengthx;

  lengthx = a.length;

  out = [];
  out.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    if(a[i] || b[i]){
      if( !(a[i] && b[i]) ){
        out[i] = true;
      }
    }
  }
  return out;
}
function BooleanNot(a){
  var out;
  var i, lengthx;

  lengthx = a.length;

  out = [];
  out.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    out[i] =  !a[i] ;
  }
  return out;
}
function ShiftBitsRight4Byte(w, n){
  var wb;
  var ob;
  var i, it;
  var f;
  f = false;

  if(n == 0){
    ob = w;
  }else{
    wb = w;
    ob = [];
    ob.length = 32;

    for(i = 0; i < 32; i = i + 1){
      it = i - n;

      if(it < 0){
        f = false;
      }else{
        f = wb[it];
      }

      ob[i] = f;
    }
  }

  return ob;
}
function ReadNextBit(data, nextbit){
  var bytenr, bitnumber, bit, b;

  bytenr = Math.floor(nextbit.numberValue/8);
  bitnumber = nextbit.numberValue%8;

  b = data[bytenr];

  bit = Math.floor(b/Math.pow(2, bitnumber))%2;

  nextbit.numberValue = nextbit.numberValue + 1;

  return bit;
}
function BitExtract(b, fromInc, toInc){
  return Math.floor(b/Math.pow(2, fromInc))%Math.pow(2, toInc + 1 - fromInc);
}
function ReadBitRange(data, nextbit, lengthx){
  var startbyte, endbyte;
  var startbit, endbit;
  var number, i;

  number = 0;

  startbyte = Math.floor(nextbit.numberValue/8);
  endbyte = Math.floor((nextbit.numberValue + lengthx)/8);

  startbit = nextbit.numberValue%8;
  endbit = (nextbit.numberValue + lengthx - 1)%8;

  if(startbyte == endbyte){
    number = BitExtract(data[startbyte], startbit, endbit);
  }

  nextbit.numberValue = nextbit.numberValue + lengthx;

  return number;
}
function SkipToBoundary(nextbit){
  var skip;

  skip = 8 - nextbit.numberValue%8;
  nextbit.numberValue = nextbit.numberValue + skip;
}
function ReadNextByteBoundary(data, nextbit){
  var bytenr, b;

  bytenr = Math.floor(nextbit.numberValue/8);
  b = data[bytenr];
  nextbit.numberValue = nextbit.numberValue + 8;

  return b;
}
function Read2bytesByteBoundary(data, nextbit){
  var r;

  r = 0;
  r = r + Math.pow(2, 8)*ReadNextByteBoundary(data, nextbit);
  r = r + ReadNextByteBoundary(data, nextbit);

  return r;
}
function arraysStringToNumberArray(string){
  var i;
  var array;

  array = [];
  array.length = string.length;

  for(i = 0; i < string.length; i = i + 1){
    array[i] = string[i].charCodeAt(0);
  }
  return array;
}
function arraysNumberArrayToString(array){
  var i;
  var string;

  string = [];
  string.length = array.length;

  for(i = 0; i < array.length; i = i + 1){
    string[i] = String.fromCharCode(array[i]);
  }
  return string;
}
function arraysNumberArraysEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.length == b.length){
    for(i = 0; i < a.length && equal; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function arraysBooleanArraysEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.length == b.length){
    for(i = 0; i < a.length && equal; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function arraysStringsEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.length == b.length){
    for(i = 0; i < a.length && equal; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function arraysFillNumberArray(a, value){
  var i;

  for(i = 0; i < a.length; i = i + 1){
    a[i] = value;
  }
}
function arraysFillString(a, value){
  var i;

  for(i = 0; i < a.length; i = i + 1){
    a[i] = value;
  }
}
function arraysFillBooleanArray(a, value){
  var i;

  for(i = 0; i < a.length; i = i + 1){
    a[i] = value;
  }
}
function arraysFillNumberArrayRange(a, value, from, to){
  var i, lengthx;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    for(i = 0; i < lengthx; i = i + 1){
      a[from + i] = value;
    }

    success = true;
  }else{
    success = false;
  }

  return success;
}
function arraysFillBooleanArrayRange(a, value, from, to){
  var i, lengthx;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    for(i = 0; i < lengthx; i = i + 1){
      a[from + i] = value;
    }

    success = true;
  }else{
    success = false;
  }

  return success;
}
function arraysFillStringRange(a, value, from, to){
  var i, lengthx;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    for(i = 0; i < lengthx; i = i + 1){
      a[from + i] = value;
    }

    success = true;
  }else{
    success = false;
  }

  return success;
}
function arraysCopyNumberArray(a){
  var i;
  var n;

  n = [];
  n.length = a.length;

  for(i = 0; i < a.length; i = i + 1){
    n[i] = a[i];
  }

  return n;
}
function arraysCopyBooleanArray(a){
  var i;
  var n;

  n = [];
  n.length = a.length;

  for(i = 0; i < a.length; i = i + 1){
    n[i] = a[i];
  }

  return n;
}
function arraysCopyString(a){
  var i;
  var n;

  n = [];
  n.length = a.length;

  for(i = 0; i < a.length; i = i + 1){
    n[i] = a[i];
  }

  return n;
}
function arraysCopyNumberArrayRange(a, from, to, copyReference){
  var i, lengthx;
  var n;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;

    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a[from + i];
    }

    copyReference.numberArray = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function arraysCopyBooleanArrayRange(a, from, to, copyReference){
  var i, lengthx;
  var n;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;

    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a[from + i];
    }

    copyReference.booleanArray = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function arraysCopyStringRange(a, from, to, copyReference){
  var i, lengthx;
  var n;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;

    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a[from + i];
    }

    copyReference.string = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function arraysIsLastElement(lengthx, index){
  return index + 1 == lengthx;
}
function arraysCreateNumberArray(lengthx, value){
  var array;

  array = [];
  array.length = lengthx;
  arraysFillNumberArray(array, value);

  return array;
}
function arraysCreateBooleanArray(lengthx, value){
  var array;

  array = [];
  array.length = lengthx;
  arraysFillBooleanArray(array, value);

  return array;
}
function arraysCreateString(lengthx, value){
  var array;

  array = [];
  array.length = lengthx;
  arraysFillString(array, value);

  return array;
}
function arraysSwapElementsOfNumberArray(A, ai, bi){
  var tmp;

  tmp = A[ai];
  A[ai] = A[bi];
  A[bi] = tmp;
}
function arraysSwapElementsOfStringArray(A, ai, bi){
  var tmp;

  tmp = A.stringArray[ai];
  A.stringArray[ai] = A.stringArray[bi];
  A.stringArray[bi] = tmp;
}
function arraysReverseNumberArray(array){
  var i;

  for(i = 0; i < array.length/2; i = i + 1){
    arraysSwapElementsOfNumberArray(array, i, array.length - i - 1);
  }
}


module.exports = {
	ZLibCompressNoCompression: ZLibCompressNoCompression,
	ZLibCompressStaticHuffman: ZLibCompressStaticHuffman,
	ComputeAdler32: ComputeAdler32,
	Pack: Pack,
	Unpack: Unpack,
	DeflateDataStaticHuffman: DeflateDataStaticHuffman,
	FindMatch: FindMatch,
	GenerateBitReverseLookupTable: GenerateBitReverseLookupTable,
	ReverseBits: ReverseBits,
	DeflateDataNoCompression: DeflateDataNoCompression,
	GetDeflateStaticHuffmanCode: GetDeflateStaticHuffmanCode,
	GetDeflateLengthCode: GetDeflateLengthCode,
	GetDeflateDistanceCode: GetDeflateDistanceCode,
	AppendBitsToBytesLeft: AppendBitsToBytesLeft,
	AppendBitsToBytesRight: AppendBitsToBytesRight,
	ReadXbytes: ReadXbytes,
	Read4bytesBE: Read4bytesBE,
	Read2bytesBE: Read2bytesBE,
	ReadByte: ReadByte,
	Read4bytesLE: Read4bytesLE,
	WriteByte: WriteByte,
	Write2BytesLE: Write2BytesLE,
	Write4BytesLE: Write4BytesLE,
	Write2BytesBE: Write2BytesBE,
	Write4BytesBE: Write4BytesBE,
	WriteStringBytes: WriteStringBytes,
	BytesRound: BytesRound,
	ByteArrayToNumberArray: ByteArrayToNumberArray,
	NumberArrayToByteArray: NumberArrayToByteArray,
	ByteArraysEqual: ByteArraysEqual,
	CopyByteArray: CopyByteArray,
	ByteArrayLength: ByteArrayLength,
	CreateAndFillByteArray: CreateAndFillByteArray,
	CreateByteArray: CreateByteArray,
	SetByte: SetByte,
	GetByte: GetByte,
	AssertByteArraysEqual: AssertByteArraysEqual,
	FreeByteArray: FreeByteArray,
	CopyByteArrayRange: CopyByteArrayRange,
	BytesToTextBase16: BytesToTextBase16,
	TextToBytesBase16: TextToBytesBase16,
	FreeBase64ByteCombinations: FreeBase64ByteCombinations,
	GenerateBase16ByteCombinations: GenerateBase16ByteCombinations,
	Negate: Negate,
	Positive: Positive,
	Factorial: Factorial,
	Round: Round,
	BankersRound: BankersRound,
	Ceil: Ceil,
	Floor: Floor,
	Truncate: Truncate,
	Absolute: Absolute,
	Logarithm: Logarithm,
	NaturalLogarithm: NaturalLogarithm,
	Sin: Sin,
	Cos: Cos,
	Tan: Tan,
	Asin: Asin,
	Acos: Acos,
	Atan: Atan,
	Atan2: Atan2,
	Squareroot: Squareroot,
	Exp: Exp,
	DivisibleBy: DivisibleBy,
	Combinations: Combinations,
	Permutations: Permutations,
	EpsilonCompare: EpsilonCompare,
	GreatestCommonDivisor: GreatestCommonDivisor,
	GCDWithSubtraction: GCDWithSubtraction,
	IsInteger: IsInteger,
	GreatestCommonDivisorWithCheck: GreatestCommonDivisorWithCheck,
	LeastCommonMultiple: LeastCommonMultiple,
	Sign: Sign,
	Max: Max,
	Min: Min,
	Power: Power,
	Gamma: Gamma,
	LogGamma: LogGamma,
	LanczosApproximation: LanczosApproximation,
	Beta: Beta,
	Sinh: Sinh,
	Cosh: Cosh,
	Tanh: Tanh,
	Cot: Cot,
	Sec: Sec,
	Csc: Csc,
	Coth: Coth,
	Sech: Sech,
	Csch: Csch,
	Error: Error,
	ErrorInverse: ErrorInverse,
	FallingFactorial: FallingFactorial,
	RisingFactorial: RisingFactorial,
	Hypergeometric: Hypergeometric,
	HypergeometricDirect: HypergeometricDirect,
	BernouilliNumber: BernouilliNumber,
	AkiyamaTanigawaAlgorithm: AkiyamaTanigawaAlgorithm,
	CreateBooleanReference: CreateBooleanReference,
	CreateBooleanArrayReference: CreateBooleanArrayReference,
	CreateBooleanArrayReferenceLengthValue: CreateBooleanArrayReferenceLengthValue,
	FreeBooleanArrayReference: FreeBooleanArrayReference,
	CreateCharacterReference: CreateCharacterReference,
	CreateNumberReference: CreateNumberReference,
	CreateNumberArrayReference: CreateNumberArrayReference,
	CreateNumberArrayReferenceLengthValue: CreateNumberArrayReferenceLengthValue,
	FreeNumberArrayReference: FreeNumberArrayReference,
	CreateStringReference: CreateStringReference,
	CreateStringReferenceLengthValue: CreateStringReferenceLengthValue,
	FreeStringReference: FreeStringReference,
	CreateStringArrayReference: CreateStringArrayReference,
	CreateStringArrayReferenceLengthValue: CreateStringArrayReferenceLengthValue,
	FreeStringArrayReference: FreeStringArrayReference,
	And4Byte: And4Byte,
	And2Byte: And2Byte,
	AndByte: AndByte,
	AndBytes: AndBytes,
	Or4Byte: Or4Byte,
	Or2Byte: Or2Byte,
	OrByte: OrByte,
	OrBytes: OrBytes,
	Xor4Byte: Xor4Byte,
	Xor2Byte: Xor2Byte,
	XorByte: XorByte,
	XorBytes: XorBytes,
	Not4Byte: Not4Byte,
	Not2Byte: Not2Byte,
	NotByte: NotByte,
	NotBytes: NotBytes,
	ShiftLeft4Byte: ShiftLeft4Byte,
	ShiftLeft2Byte: ShiftLeft2Byte,
	ShiftLeftByte: ShiftLeftByte,
	ShiftLeftBytes: ShiftLeftBytes,
	ShiftRight4Byte: ShiftRight4Byte,
	ShiftRight2Byte: ShiftRight2Byte,
	ShiftRightByte: ShiftRightByte,
	ShiftRightBytes: ShiftRightBytes,
	RotateLeft4Byte: RotateLeft4Byte,
	RotateRight4Bytes: RotateRight4Bytes,
	CreateBooleanArrayFromNumber: CreateBooleanArrayFromNumber,
	BooleanArrayToNumber: BooleanArrayToNumber,
	BooleanAnd: BooleanAnd,
	BooleanXor: BooleanXor,
	BooleanNot: BooleanNot,
	ShiftBitsRight4Byte: ShiftBitsRight4Byte,
	ReadNextBit: ReadNextBit,
	BitExtract: BitExtract,
	ReadBitRange: ReadBitRange,
	SkipToBoundary: SkipToBoundary,
	ReadNextByteBoundary: ReadNextByteBoundary,
	Read2bytesByteBoundary: Read2bytesByteBoundary,
	arraysStringToNumberArray: arraysStringToNumberArray,
	arraysNumberArrayToString: arraysNumberArrayToString,
	arraysNumberArraysEqual: arraysNumberArraysEqual,
	arraysBooleanArraysEqual: arraysBooleanArraysEqual,
	arraysStringsEqual: arraysStringsEqual,
	arraysFillNumberArray: arraysFillNumberArray,
	arraysFillString: arraysFillString,
	arraysFillBooleanArray: arraysFillBooleanArray,
	arraysFillNumberArrayRange: arraysFillNumberArrayRange,
	arraysFillBooleanArrayRange: arraysFillBooleanArrayRange,
	arraysFillStringRange: arraysFillStringRange,
	arraysCopyNumberArray: arraysCopyNumberArray,
	arraysCopyBooleanArray: arraysCopyBooleanArray,
	arraysCopyString: arraysCopyString,
	arraysCopyNumberArrayRange: arraysCopyNumberArrayRange,
	arraysCopyBooleanArrayRange: arraysCopyBooleanArrayRange,
	arraysCopyStringRange: arraysCopyStringRange,
	arraysIsLastElement: arraysIsLastElement,
	arraysCreateNumberArray: arraysCreateNumberArray,
	arraysCreateBooleanArray: arraysCreateBooleanArray,
	arraysCreateString: arraysCreateString,
	arraysSwapElementsOfNumberArray: arraysSwapElementsOfNumberArray,
	arraysSwapElementsOfStringArray: arraysSwapElementsOfStringArray,
	arraysReverseNumberArray: arraysReverseNumberArray,
	test: test
};